/*
 * This file is a part of morm
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef headerfile_morm_dbexpression
#define headerfile_morm_dbexpression

#include <vector>
#include "baseexpression.h"
#include "morm_types.h"


namespace morm
{

class DbExpression : public BaseExpression
{
public:

	DbExpression();
	virtual ~DbExpression();

	virtual void set_output_type(int output_type);
	virtual int  get_output_type();


	virtual void prepare_to_where_clause();

	virtual DbExpression & group_or(PT::TextStream & stream);
	virtual DbExpression & group_and(PT::TextStream & stream);
	virtual DbExpression & group_end(PT::TextStream & stream);

	virtual DbExpression & page(PT::TextStream & stream, size_t page_number, size_t page_size);

	template<typename FieldValue>
	void add_field_for_select(const wchar_t * new_column_expression, const wchar_t * new_column_name, FieldValue & field_value)
	{
		std::wstring column_expression; // field() methods can be called recursively, so don't make it as class object

		column_expression = new_column_expression;
		column_expression += L" as ";
		column_expression += new_column_name;

		field(column_expression.c_str(), field_value, false, false, false, false);
	}


protected:

	int output_type;
	std::vector<int> conjunctions;


	bool can_field_be_generated(bool insertable, bool updatable, bool is_primary_key);

	void field_before();

	void before_field_value(const std::wstring &);
	void after_field_value(const std::wstring &);

	void before_field_value(const std::string &);
	void after_field_value(const std::string &);

	void before_field_value(const wchar_t *);
	void after_field_value(const wchar_t *);

	void before_field_value(const char *);
	void after_field_value(const char *);

	void before_field_value(const PT::Date &);
	void after_field_value(const PT::Date &);

	void put_name_value_separator();


private:

	void before_field_value_string();
	void after_field_value_string();


};

}

#endif
